import requests
from src.config.config_parser import Config


class Synapse:
    def __init__(self):
        self.homeserver = None
        self.access_token = None
        self.header = None
        self.base_url = None
        self.initialize()

    def initialize(self):
        config = Config()
        self.homeserver, self.access_token = config.get_config()
        self.base_url = f"https://{self.homeserver}/_synapse/admin"
        self.header = {"Authorization": f"Bearer {self.access_token}"}

    def request(self, url, method):
        try:
            return requests.request(url=url, headers=self.header, method=method)
        except Exception as e:
            print(f"Error making request: {e}")
            return None

    def delete_media_by_id(self, media_id):
        url = f"{self.base_url}/v1/media/{self.homeserver}/{media_id}"
        return self.request(url, "DELETE")

    def get_users_media_usage(self, limit=100, order="media_length", direction="b"):
        url = f"{self.base_url}/v1/statistics/users/media?limit={limit}&order_by={order}&dir={direction}"
        return self.request(url, "GET")

    def search_for_user(self, search_term, limit=10, order="media_length", direction="b"):
        url = f"{self.base_url}/v1/statistics/users/media?search_term={search_term}&order_by={order}&dir={direction}"
        return self.request(url, "GET")

    def get_rooms_list(self, room_name=None, batch=0):
        if room_name:
            url = f"{self.base_url}/v1/rooms?search_term={room_name}&from={batch}"
        else:
            url = f"{self.base_url}/v1/rooms?from={batch}"
        return self.request(url, "GET")

    def get_user_info(self, user_id):
        url = f"{self.base_url}/v2/users/{user_id}"
        return self.request(url, "GET")

    def get_user_media(self, user_id, limit=100, order="media_length", direction="b"):
        url = f"{self.base_url}/v1/users/{user_id}/media?limit={limit}&order_by={order}&dir={direction}"
        return self.request(url, "GET")